### Name: ccr12.coef
### Title: Computes the $ccr_1^2$ coefficient for variable subset selection
### Aliases: ccr12.coef
### Keywords: manip

### ** Examples


## 1) A Linear Discriminant Analysis example with a very small data set. 
## We considered the Iris data and three groups, 
## defined by species (setosa, versicolor and virginica).
## Both the 'iris' and the 'iris3' R objects are used, for convenience in  
## computing the Total matrix (T=irismat) and the Effects matrix (H=irish).

data(iris)
data(iris3) 
irismat <-149*var(iris[,1:4])
weights<-c(49,49,49)
irish <- irismat - matrix(weights%*% t(apply(iris3,3,var)),4,4)
ccr12.coef(irismat,H=irish,r=2,c(1,3))
## [1]  0.9589055

## ---------------------------------------------------------------

## 2) An example computing the value of the ccr1_2 criteria for two  
## subsets produced when the anneal function attempted to optimize
## the zeta_2 criterion (using an absurdly small number of iterations).

zetaresults<-anneal(irismat,2,nsol=2,niter=2,criterion="zeta2",H=irish,r=2)
ccr12.coef(irismat,H=irish,r=2,zetaresults$subsets)

##              Card.2
##Solution 1 0.9526304
##Solution 2 0.9558787

## ---------------------------------------------------------------




